@@ -6,46 +6,28 @@
 
 const { CALL, CONSTRUCT, READ, ReferenceTracker } = require("eslint-utils")
 const enumeratePropertyNames = require("../util/enumerate-property-names")
+const getConfiguredNodeVersion = require("../util/get-configured-node-version")
+const getSemverRange = require("../util/get-semver-range")
 
 const modules = {
     _linklist: {
         [READ]: { since: "5.0.0", replacedBy: null },
     },
-    assert: {
-        deepEqual: {
-            [READ]: {
-                since: "10.0.0",
-                replacedBy:
-                    "'assert.deepStrictEqual' or 'assert.strict.deepEqual'",
-            },
-        },
-        equal: {
-            [READ]: {
-                since: "10.0.0",
-                replacedBy: "'assert.strictEqual' or 'assert.strict.equal'",
-            },
-        },
-        notDeepEqual: {
-            [READ]: {
-                since: "10.0.0",
-                replacedBy:
-                    "'assert.notDeepStrictEqual' or 'assert.strict.notDeepEqual'",
-            },
-        },
-        notEqual: {
-            [READ]: {
-                since: "10.0.0",
-                replacedBy:
-                    "'assert.notStrictEqual' or 'assert.strict.notEqual'",
-            },
-        },
+    //eslint-disable-next-line camelcase
+    _stream_wrap: {
+        [READ]: { since: "12.0.0", replacedBy: null },
     },
     //eslint-disable-next-line camelcase
     async_hooks: {
         currentId: {
             [READ]: {
                 since: "8.2.0",
-                replacedBy: "'async_hooks.executionAsyncId()'",
+                replacedBy: [
+                    {
+                        name: "'async_hooks.executionAsyncId()'",
+                        supported: "8.1.0",
+                    },
+                ],
             },
         },
         triggerId: {
@@ -59,17 +41,28 @@ const modules = {
         Buffer: {
             [CONSTRUCT]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.alloc()' or 'buffer.Buffer.from()'",
+                replacedBy: [
+                    { name: "'buffer.Buffer.alloc()'", supported: "5.10.0" },
+                    { name: "'buffer.Buffer.from()'", supported: "5.10.0" },
+                ],
             },
             [CALL]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.alloc()' or 'buffer.Buffer.from()'",
+                replacedBy: [
+                    { name: "'buffer.Buffer.alloc()'", supported: "5.10.0" },
+                    { name: "'buffer.Buffer.from()'", supported: "5.10.0" },
+                ],
             },
         },
         SlowBuffer: {
             [READ]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.allocUnsafeSlow()'",
+                replacedBy: [
+                    {
+                        name: "'buffer.Buffer.allocUnsafeSlow()'",
+                        supported: "5.12.0",
+                    },
+                ],
             },
         },
     },
@@ -92,35 +85,67 @@ const modules = {
         createCipher: {
             [READ]: {
                 since: "10.0.0",
-                replacedBy: "'crypto.createCipheriv()'",
+                replacedBy: [
+                    { name: "'crypto.createCipheriv()'", supported: "0.1.94" },
+                ],
             },
         },
         createCredentials: {
             [READ]: {
                 since: "0.12.0",
-                replacedBy: "'tls.createSecureContext()'",
+                replacedBy: [
+                    {
+                        name: "'tls.createSecureContext()'",
+                        supported: "0.11.13",
+                    },
+                ],
             },
         },
         createDecipher: {
             [READ]: {
                 since: "10.0.0",
-                replacedBy: "'crypto.createDecipheriv()'",
+                replacedBy: [
+                    {
+                        name: "'crypto.createDecipheriv()'",
+                        supported: "0.1.94",
+                    },
+                ],
             },
         },
         fips: {
             [READ]: {
                 since: "10.0.0",
-                replacedBy: "'crypto.getFips()' and 'crypto.setFips()'",
+                replacedBy: [
+                    {
+                        name: "'crypto.getFips()' and 'crypto.setFips()'",
+                        supported: "10.0.0",
+                    },
+                ],
             },
         },
         prng: {
-            [READ]: { since: "11.0.0", replacedBy: "'crypto.randomBytes()'" },
+            [READ]: {
+                since: "11.0.0",
+                replacedBy: [
+                    { name: "'crypto.randomBytes()'", supported: "0.5.8" },
+                ],
+            },
         },
         pseudoRandomBytes: {
-            [READ]: { since: "11.0.0", replacedBy: "'crypto.randomBytes()'" },
+            [READ]: {
+                since: "11.0.0",
+                replacedBy: [
+                    { name: "'crypto.randomBytes()'", supported: "0.5.8" },
+                ],
+            },
         },
         rng: {
-            [READ]: { since: "11.0.0", replacedBy: "'crypto.randomBytes()'" },
+            [READ]: {
+                since: "11.0.0",
+                replacedBy: [
+                    { name: "'crypto.randomBytes()'", supported: "0.5.8" },
+                ],
+            },
         },
     },
     domain: {
@@ -131,14 +156,24 @@ const modules = {
             listenerCount: {
                 [READ]: {
                     since: "4.0.0",
-                    replacedBy: "'events.EventEmitter#listenerCount()'",
+                    replacedBy: [
+                        {
+                            name: "'events.EventEmitter#listenerCount()'",
+                            supported: "3.2.0",
+                        },
+                    ],
                 },
             },
         },
         listenerCount: {
             [READ]: {
                 since: "4.0.0",
-                replacedBy: "'events.EventEmitter#listenerCount()'",
+                replacedBy: [
+                    {
+                        name: "'events.EventEmitter#listenerCount()'",
+                        supported: "3.2.0",
+                    },
+                ],
             },
         },
     },
@@ -152,7 +187,10 @@ const modules = {
         exists: {
             [READ]: {
                 since: "4.0.0",
-                replacedBy: "'fs.stat()' or 'fs.access()'",
+                replacedBy: [
+                    { name: "'fs.stat()'", supported: "0.0.2" },
+                    { name: "'fs.access()'", supported: "0.11.15" },
+                ],
             },
         },
         lchmod: {
@@ -161,60 +199,120 @@ const modules = {
         lchmodSync: {
             [READ]: { since: "0.4.0", replacedBy: null },
         },
-        lchown: {
-            [READ]: { since: "0.4.0", replacedBy: null },
-        },
-        lchownSync: {
-            [READ]: { since: "0.4.0", replacedBy: null },
-        },
     },
     http: {
         createClient: {
-            [READ]: { since: "0.10.0", replacedBy: "'http.request()'" },
+            [READ]: {
+                since: "0.10.0",
+                replacedBy: [{ name: "'http.request()'", supported: "0.3.6" }],
+            },
         },
     },
     module: {
         Module: {
+            createRequireFromPath: {
+                [READ]: {
+                    since: "12.2.0",
+                    replacedBy: [
+                        {
+                            name: "'module.createRequire()'",
+                            supported: "12.2.0",
+                        },
+                    ],
+                },
+            },
             requireRepl: {
-                [READ]: { since: "6.0.0", replacedBy: "'require(\"repl\")'" },
+                [READ]: {
+                    since: "6.0.0",
+                    replacedBy: "'require(\"repl\")'",
+                },
             },
             _debug: {
                 [READ]: { since: "9.0.0", replacedBy: null },
             },
         },
+        createRequireFromPath: {
+            [READ]: {
+                since: "12.2.0",
+                replacedBy: [
+                    {
+                        name: "'module.createRequire()'",
+                        supported: "12.2.0",
+                    },
+                ],
+            },
+        },
         requireRepl: {
-            [READ]: { since: "6.0.0", replacedBy: "'require(\"repl\")'" },
+            [READ]: {
+                since: "6.0.0",
+                replacedBy: "'require(\"repl\")'",
+            },
         },
         _debug: {
             [READ]: { since: "9.0.0", replacedBy: null },
         },
     },
+    net: {
+        _setSimultaneousAccepts: {
+            [READ]: { since: "12.0.0", replacedBy: null },
+        },
+    },
     os: {
         getNetworkInterfaces: {
-            [READ]: { since: "0.6.0", replacedBy: "'os.networkInterfaces()'" },
+            [READ]: {
+                since: "0.6.0",
+                replacedBy: [
+                    { name: "'os.networkInterfaces()'", supported: "0.6.0" },
+                ],
+            },
         },
         tmpDir: {
-            [READ]: { since: "7.0.0", replacedBy: "'os.tmpdir()'" },
+            [READ]: {
+                since: "7.0.0",
+                replacedBy: [{ name: "'os.tmpdir()'", supported: "0.9.9" }],
+            },
         },
     },
     path: {
         _makeLong: {
-            [READ]: { since: "9.0.0", replacedBy: "'path.toNamespacedPath()'" },
+            [READ]: {
+                since: "9.0.0",
+                replacedBy: [
+                    { name: "'path.toNamespacedPath()'", supported: "9.0.0" },
+                ],
+            },
         },
     },
     process: {
         EventEmitter: {
-            [READ]: { since: "0.6.0", replacedBy: "'require(\"events\")'" },
+            [READ]: {
+                since: "0.6.0",
+                replacedBy: "'require(\"events\")'",
+            },
         },
         assert: {
-            [READ]: { since: "10.0.0", replacedBy: "'require(\"assert\")'" },
+            [READ]: {
+                since: "10.0.0",
+                replacedBy: "'require(\"assert\")'",
+            },
         },
         binding: {
             [READ]: { since: "10.9.0", replacedBy: null },
         },
         env: {
             NODE_REPL_HISTORY_FILE: {
-                [READ]: { since: "4.0.0", replacedBy: "'NODE_REPL_HISTORY'" },
+                [READ]: {
+                    since: "4.0.0",
+                    replacedBy: "'NODE_REPL_HISTORY'",
+                },
+            },
+        },
+        report: {
+            triggerReport: {
+                [READ]: {
+                    since: "11.12.0",
+                    replacedBy: "'process.report.writeReport()'",
+                },
             },
         },
     },
@@ -244,34 +342,54 @@ const modules = {
         Buffer: {
             [CONSTRUCT]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.alloc()' or 'buffer.Buffer.from()'",
+                replacedBy: [
+                    { name: "'buffer.Buffer.alloc()'", supported: "5.10.0" },
+                    { name: "'buffer.Buffer.from()'", supported: "5.10.0" },
+                ],
             },
             [CALL]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.alloc()' or 'buffer.Buffer.from()'",
+                replacedBy: [
+                    { name: "'buffer.Buffer.alloc()'", supported: "5.10.0" },
+                    { name: "'buffer.Buffer.from()'", supported: "5.10.0" },
+                ],
             },
         },
         SlowBuffer: {
             [READ]: {
                 since: "6.0.0",
-                replacedBy: "'buffer.Buffer.allocUnsafeSlow()'",
+                replacedBy: [
+                    {
+                        name: "'buffer.Buffer.allocUnsafeSlow()'",
+                        supported: "5.12.0",
+                    },
+                ],
             },
         },
     },
     sys: {
-        [READ]: { since: "0.3.0", replacedBy: "'util' module" },
+        [READ]: {
+            since: "0.3.0",
+            replacedBy: "'util' module",
+        },
     },
     timers: {
         enroll: {
             [READ]: {
                 since: "10.0.0",
-                replacedBy: "'setTimeout()' or 'setInterval()'",
+                replacedBy: [
+                    { name: "'setTimeout()'", supported: "0.0.1" },
+                    { name: "'setInterval()'", supported: "0.0.1" },
+                ],
             },
         },
         unenroll: {
             [READ]: {
                 since: "10.0.0",
-                replacedBy: "'clearTimeout()' or 'clearInterval()'",
+                replacedBy: [
+                    { name: "'clearTimeout()'", supported: "0.0.1" },
+                    { name: "'clearInterval()'", supported: "0.0.1" },
+                ],
             },
         },
     },
@@ -280,19 +398,33 @@ const modules = {
             [READ]: { since: "0.10.0", replacedBy: null },
         },
         CryptoStream: {
-            [READ]: { since: "0.12.0", replacedBy: "'tls.TLSSocket'" },
+            [READ]: {
+                since: "0.12.0",
+                replacedBy: [{ name: "'tls.TLSSocket'", supported: "0.11.4" }],
+            },
         },
         SecurePair: {
-            [READ]: { since: "6.0.0", replacedBy: "'tls.TLSSocket'" },
+            [READ]: {
+                since: "6.0.0",
+                replacedBy: [{ name: "'tls.TLSSocket'", supported: "0.11.4" }],
+            },
         },
         convertNPNProtocols: {
             [READ]: { since: "10.0.0", replacedBy: null },
         },
         createSecurePair: {
-            [READ]: { since: "6.0.0", replacedBy: "'tls.TLSSocket'" },
+            [READ]: {
+                since: "6.0.0",
+                replacedBy: [{ name: "'tls.TLSSocket'", supported: "0.11.4" }],
+            },
         },
         parseCertString: {
-            [READ]: { since: "8.6.0", replacedBy: "'querystring.parse()'" },
+            [READ]: {
+                since: "8.6.0",
+                replacedBy: [
+                    { name: "'querystring.parse()'", supported: "0.1.25" },
+                ],
+            },
         },
     },
     tty: {
@@ -306,27 +438,57 @@ const modules = {
     },
     url: {
         parse: {
-            [READ]: { since: "11.0.0", replacedBy: "'url.URL' constructor" },
+            [READ]: {
+                since: "11.0.0",
+                replacedBy: [
+                    { name: "'url.URL' constructor", supported: "6.13.0" },
+                ],
+            },
         },
         resolve: {
-            [READ]: { since: "11.0.0", replacedBy: "'url.URL' constructor" },
+            [READ]: {
+                since: "11.0.0",
+                replacedBy: [
+                    { name: "'url.URL' constructor", supported: "6.13.0" },
+                ],
+            },
         },
     },
     util: {
         debug: {
-            [READ]: { since: "0.12.0", replacedBy: "'console.error()'" },
+            [READ]: {
+                since: "0.12.0",
+                replacedBy: [
+                    { name: "'console.error()'", supported: "0.1.100" },
+                ],
+            },
         },
         error: {
-            [READ]: { since: "0.12.0", replacedBy: "'console.error()'" },
+            [READ]: {
+                since: "0.12.0",
+                replacedBy: [
+                    { name: "'console.error()'", supported: "0.1.100" },
+                ],
+            },
         },
         isArray: {
-            [READ]: { since: "4.0.0", replacedBy: "'Array.isArray()'" },
+            [READ]: {
+                since: "4.0.0",
+                replacedBy: [
+                    { name: "'Array.isArray()'", supported: "0.1.100" },
+                ],
+            },
         },
         isBoolean: {
             [READ]: { since: "4.0.0", replacedBy: null },
         },
         isBuffer: {
-            [READ]: { since: "4.0.0", replacedBy: "'Buffer.isBuffer()'" },
+            [READ]: {
+                since: "4.0.0",
+                replacedBy: [
+                    { name: "'Buffer.isBuffer()'", supported: "0.1.101" },
+                ],
+            },
         },
         isDate: {
             [READ]: { since: "4.0.0", replacedBy: null },
@@ -368,16 +530,30 @@ const modules = {
             [READ]: { since: "6.0.0", replacedBy: "a third party module" },
         },
         print: {
-            [READ]: { since: "0.12.0", replacedBy: "'console.log()'" },
+            [READ]: {
+                since: "0.12.0",
+                replacedBy: [{ name: "'console.log()'", supported: "0.1.100" }],
+            },
         },
         pump: {
-            [READ]: { since: "0.10.0", replacedBy: "'stream.Readable#pipe()'" },
+            [READ]: {
+                since: "0.10.0",
+                replacedBy: [
+                    { name: "'stream.Readable#pipe()'", supported: "0.9.4" },
+                ],
+            },
         },
         puts: {
-            [READ]: { since: "0.12.0", replacedBy: "'console.log()'" },
+            [READ]: {
+                since: "0.12.0",
+                replacedBy: [{ name: "'console.log()'", supported: "0.1.100" }],
+            },
         },
         _extend: {
-            [READ]: { since: "6.0.0", replacedBy: "'Object.assign()'" },
+            [READ]: {
+                since: "6.0.0",
+                replacedBy: [{ name: "'Object.assign()'", supported: "4.0.0" }],
+            },
         },
     },
     vm: {
@@ -390,11 +566,17 @@ const globals = {
     Buffer: {
         [CONSTRUCT]: {
             since: "6.0.0",
-            replacedBy: "'Buffer.alloc()' or 'Buffer.from()'",
+            replacedBy: [
+                { name: "'Buffer.alloc()'", supported: "5.10.0" },
+                { name: "'Buffer.from()'", supported: "5.10.0" },
+            ],
         },
         [CALL]: {
             since: "6.0.0",
-            replacedBy: "'Buffer.alloc()' or 'Buffer.from()'",
+            replacedBy: [
+                { name: "'Buffer.alloc()'", supported: "5.10.0" },
+                { name: "'Buffer.from()'", supported: "5.10.0" },
+            ],
         },
     },
     COUNTER_NET_SERVER_CONNECTION: {
@@ -416,7 +598,10 @@ const globals = {
         [READ]: { since: "11.0.0", replacedBy: null },
     },
     GLOBAL: {
-        [READ]: { since: "6.0.0", replacedBy: "'global'" },
+        [READ]: {
+            since: "6.0.0",
+            replacedBy: [{ name: "'global'", supported: "0.1.27" }],
+        },
     },
     Intl: {
         v8BreakIterator: {
@@ -432,7 +617,10 @@ const globals = {
         },
     },
     root: {
-        [READ]: { since: "6.0.0", replacedBy: "'global'" },
+        [READ]: {
+            since: "6.0.0",
+            replacedBy: [{ name: "'global'", supported: "0.1.27" }],
+        },
     },
     process: modules.process,
 }
@@ -440,11 +628,24 @@ const globals = {
 /**
  * Makes a replacement message.
  *
- * @param {string|null} replacedBy - The text of substitute way.
+ * @param {string|array|null} replacedBy - The text of substitute way.
+ * @param {Range} version - The configured version range
  * @returns {string} Replacement message.
  */
-function toReplaceMessage(replacedBy) {
-    return replacedBy ? `. Use ${replacedBy} instead` : ""
+function toReplaceMessage(replacedBy, version) {
+    let message = replacedBy
+
+    if (Array.isArray(replacedBy)) {
+        message = replacedBy
+            .filter(
+                ({ supported }) =>
+                    !version.intersects(getSemverRange(`<${supported}`))
+            )
+            .map(({ name }) => name)
+            .join(" or ")
+    }
+
+    return message ? `. Use ${message} instead` : ""
 }
 
 /**
@@ -462,6 +663,22 @@ function toName(type, path) {
             : baseName
 }
 
+/**
+ * Parses the options.
+ * @param {RuleContext} context The rule context.
+ * @returns {{version:Range,ignoredGlobalItems:Set<string>,ignoredModuleItems:Set<string>}} Parsed
+ *     value.
+ */
+function parseOptions(context) {
+    const raw = context.options[0] || {}
+    const filePath = context.getFilename()
+    const version = getConfiguredNodeVersion(raw.version, filePath)
+    const ignoredModuleItems = new Set(raw.ignoreModuleItems || [])
+    const ignoredGlobalItems = new Set(raw.ignoreGlobalItems || [])
+
+    return Object.freeze({ version, ignoredGlobalItems, ignoredModuleItems })
+}
+
 module.exports = {
     meta: {
         docs: {
@@ -469,7 +686,7 @@ module.exports = {
             category: "Best Practices",
             recommended: true,
             url:
-                "https://github.com/mysticatea/eslint-plugin-node/blob/v8.0.1/docs/rules/no-deprecated-api.md",
+                "https://github.com/mysticatea/eslint-plugin-node/blob/v11.1.0/docs/rules/no-deprecated-api.md",
         },
         type: "problem",
         fixable: null,
@@ -477,6 +694,9 @@ module.exports = {
             {
                 type: "object",
                 properties: {
+                    version: {
+                        type: "string",
+                    },
                     ignoreModuleItems: {
                         type: "array",
                         items: {
@@ -502,9 +722,11 @@ module.exports = {
         ],
     },
     create(context) {
-        const options = context.options[0] || {}
-        const ignoredModuleItems = new Set(options.ignoreModuleItems || [])
-        const ignoredGlobalItems = new Set(options.ignoreGlobalItems || [])
+        const {
+            ignoredModuleItems,
+            ignoredGlobalItems,
+            version,
+        } = parseOptions(context)
 
         /**
          * Reports a use of a deprecated API.
@@ -523,7 +745,7 @@ module.exports = {
                 data: {
                     name,
                     version: info.since,
-                    replace: toReplaceMessage(info.replacedBy),
+                    replace: toReplaceMessage(info.replacedBy, version),
                 },
             })
         }
