@@ -4,6 +4,6 @@
  */
 "use strict"
 
-require("./update-lib-configs-recommended")
+require("./update-docs-header-and-footer")
 require("./update-lib-index")
 require("./update-readme")
