@@ -4,8 +4,8 @@
  */
 "use strict"
 
-const RuleTester = require("eslint").RuleTester
-const { configs } = require("@mysticatea/eslint-plugin")
+const { RuleTester } = require("eslint")
+const { builtin } = require("globals")
 const rule = require("../../../../lib/rules/no-unsupported-features/es-builtins")
 
 /**
@@ -56,7 +56,7 @@ function concat(patterns) {
 
 const ruleTester = new RuleTester({
     parserOptions: { ecmaVersion: 2018 },
-    globals: Object.assign({}, configs.es2015.globals, configs.es2017.globals),
+    globals: builtin,
 })
 ruleTester.run(
     "no-unsupported-features/es-builtins",
@@ -128,6 +128,53 @@ ruleTester.run(
                 },
             ],
         },
+        {
+            keyword: "BigInt",
+            valid: [
+                {
+                    code: "bigint",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "(function(BigInt) { BigInt }(b))",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "BigInt",
+                    options: [{ version: "10.4.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "BigInt",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigInt",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "(function() { BigInt })()",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigInt",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
         {
             keyword: "Map",
             valid: [
@@ -1110,6 +1157,49 @@ ruleTester.run(
                 },
             ],
         },
+        {
+            keyword: "Promise.allSettled",
+            valid: [
+                {
+                    code: "(function(Promise) { Promise.allSettled }(a))",
+                    options: [{ version: "12.8.1" }],
+                },
+                {
+                    code: "Promise.allSettled",
+                    options: [{ version: "12.9.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "Promise.allSettled",
+                    options: [{ version: "12.8.1" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "Promise.allSettled",
+                                supported: "12.9.0",
+                                version: "12.8.1",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "function wrap() { Promise.allSettled }",
+                    options: [{ version: "12.8.1" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "Promise.allSettled",
+                                supported: "12.9.0",
+                                version: "12.8.1",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
         {
             keyword: "Proxy",
             valid: [
@@ -1650,6 +1740,92 @@ ruleTester.run(
                 },
             ],
         },
+        {
+            keyword: "BigInt64Array",
+            valid: [
+                {
+                    code: "(function(BigInt64Array) { BigInt64Array }(b))",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "BigInt64Array",
+                    options: [{ version: "10.4.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "BigInt64Array",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigInt64Array",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "(function() { BigInt64Array })()",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigInt64Array",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
+        {
+            keyword: "BigUint64Array",
+            valid: [
+                {
+                    code: "(function(BigUint64Array) { BigUint64Array }(b))",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "BigUint64Array",
+                    options: [{ version: "10.4.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "BigUint64Array",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigUint64Array",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "(function() { BigUint64Array })()",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "BigUint64Array",
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
         {
             keyword: "Float32Array",
             valid: [
@@ -2052,5 +2228,48 @@ ruleTester.run(
                 },
             ],
         },
+        {
+            keyword: "globalThis",
+            valid: [
+                {
+                    code: "(function(globalThis) { globalThis }(a))",
+                    options: [{ version: "12.0.0" }],
+                },
+                {
+                    code: "globalThis",
+                    options: [{ version: "12.0.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "globalThis",
+                    options: [{ version: "11.9.9" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "globalThis",
+                                supported: "12.0.0",
+                                version: "11.9.9",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "function wrap() { globalThis }",
+                    options: [{ version: "11.9.9" }],
+                    errors: [
+                        {
+                            messageId: "unsupported",
+                            data: {
+                                name: "globalThis",
+                                supported: "12.0.0",
+                                version: "11.9.9",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
     ])
 )
