@@ -5,10 +5,17 @@
 "use strict"
 
 const path = require("path")
-const RuleTester = require("eslint").RuleTester
-const { configs } = require("@mysticatea/eslint-plugin")
+const { Linter, RuleTester } = require("eslint")
+const { builtin } = require("globals")
 const rule = require("../../../../lib/rules/no-unsupported-features/es-syntax")
 
+const ES2020Supported = (() => {
+    const config = { parserOptions: { ecmaVersion: 2020 } }
+    const messages = new Linter().verify("0n", config)
+    return messages.length === 0
+})()
+const ecmaVersion = ES2020Supported ? 2020 : 2019
+
 /**
  * Makes a file path to a fixture.
  * @param {string} name - A name.
@@ -57,7 +64,11 @@ function concat(patterns) {
         invalid: [],
     }
 
-    for (const { keyword, valid, invalid } of patterns) {
+    for (const { requiredEcmaVersion, keyword, valid, invalid } of patterns) {
+        if (requiredEcmaVersion && ecmaVersion < requiredEcmaVersion) {
+            continue
+        }
+
         ret.valid.push(...valid)
         ret.invalid.push(...invalid)
 
@@ -71,8 +82,8 @@ function concat(patterns) {
 }
 
 const ruleTester = new RuleTester({
-    parserOptions: { ecmaVersion: 2019 },
-    globals: Object.assign({}, configs.es2015.globals, configs.es2017.globals),
+    parserOptions: { ecmaVersion },
+    globals: builtin,
 })
 ruleTester.run(
     "no-unsupported-features/es-syntax",
@@ -830,6 +841,26 @@ ruleTester.run(
                     code: "exports.a = {}",
                     options: [{ version: "0.0.0" }],
                 },
+                {
+                    code: "import a from 'a'",
+                    parserOptions: { sourceType: "module" },
+                    options: [{ version: "13.1.0", ignores: ["modules"] }],
+                },
+                {
+                    code: "export default {}",
+                    parserOptions: { sourceType: "module" },
+                    options: [{ version: "13.1.0", ignores: ["modules"] }],
+                },
+                {
+                    code: "export const a = {}",
+                    parserOptions: { sourceType: "module" },
+                    options: [{ version: "13.1.0", ignores: ["modules"] }],
+                },
+                {
+                    code: "export {}",
+                    parserOptions: { sourceType: "module" },
+                    options: [{ version: "13.1.0", ignores: ["modules"] }],
+                },
             ],
             invalid: [
                 {
@@ -839,7 +870,7 @@ ruleTester.run(
                     errors: [
                         {
                             messageId: "no-modules",
-                            data: { version: "10.0.0" },
+                            data: { supported: null, version: "10.0.0" },
                         },
                     ],
                 },
@@ -850,7 +881,7 @@ ruleTester.run(
                     errors: [
                         {
                             messageId: "no-modules",
-                            data: { version: "10.0.0" },
+                            data: { supported: null, version: "10.0.0" },
                         },
                     ],
                 },
@@ -861,7 +892,7 @@ ruleTester.run(
                     errors: [
                         {
                             messageId: "no-modules",
-                            data: { version: "10.0.0" },
+                            data: { supported: null, version: "10.0.0" },
                         },
                     ],
                 },
@@ -872,7 +903,7 @@ ruleTester.run(
                     errors: [
                         {
                             messageId: "no-modules",
-                            data: { version: "10.0.0" },
+                            data: { supported: null, version: "10.0.0" },
                         },
                     ],
                 },
@@ -2376,6 +2407,112 @@ ruleTester.run(
             ],
         },
 
+        //----------------------------------------------------------------------
+        // ES2020
+        //----------------------------------------------------------------------
+        {
+            keyword: "bigint",
+            requiredEcmaVersion: 2020,
+            valid: [
+                {
+                    code: "var n = 0n",
+                    options: [{ version: "10.4.0" }],
+                },
+                {
+                    code: "var n = BigInt(0)",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "var n = new BigInt64Array()",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "var n = new BigUint64Array()",
+                    options: [{ version: "10.3.0" }],
+                },
+                {
+                    code: "var n = { [0n]: 0 }",
+                    options: [{ version: "10.4.0" }],
+                },
+                {
+                    code: "var n = class { [0n]() {} }",
+                    options: [{ version: "10.4.0" }],
+                },
+            ],
+            invalid: [
+                {
+                    code: "var n = 0n",
+                    options: [{ version: "10.3.0" }],
+                    errors: [
+                        {
+                            messageId: "no-bigint",
+                            data: {
+                                supported: "10.4.0",
+                                version: "10.3.0",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "var n = { 0n: 0 }",
+                    options: [{ version: "12.0.0" }],
+                    errors: [
+                        {
+                            messageId: "no-bigint-property-names",
+                            data: {
+                                supported: null,
+                                version: "12.0.0",
+                            },
+                        },
+                    ],
+                },
+                {
+                    code: "var n = class { 0n() {} }",
+                    options: [{ version: "12.0.0" }],
+                    errors: [
+                        {
+                            messageId: "no-bigint-property-names",
+                            data: {
+                                supported: null,
+                                version: "12.0.0",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
+        {
+            keyword: "dynamicImport",
+            requiredEcmaVersion: 2020,
+            valid: [
+                {
+                    code: "obj.import(source)",
+                    options: [{ version: "12.0.0" }],
+                },
+                {
+                    code: "import(source)",
+                    options: [
+                        { version: "13.1.0", ignores: ["dynamicImport"] },
+                    ],
+                },
+            ],
+            invalid: [
+                {
+                    code: "import(source)",
+                    options: [{ version: "13.3.0" }],
+                    errors: [
+                        {
+                            messageId: "no-dynamic-import",
+                            data: {
+                                supported: null,
+                                version: "13.3.0",
+                            },
+                        },
+                    ],
+                },
+            ],
+        },
+
         //----------------------------------------------------------------------
         // MISC
         //----------------------------------------------------------------------
@@ -2440,11 +2577,11 @@ ruleTester.run(
                 },
                 {
                     filename: fixture("invalid/a.js"),
-                    code: "var a = (b,) => 1",
+                    code: "var a = { ...obj }",
                     errors: [
                         {
-                            messageId: "no-trailing-function-commas",
-                            data: { supported: "8.0.0", version: ">=6.0.0" },
+                            messageId: "no-rest-spread-properties",
+                            data: { supported: "8.3.0", version: ">=8.0.0" },
                         },
                     ],
                 },
@@ -2460,11 +2597,11 @@ ruleTester.run(
                 },
                 {
                     filename: fixture("nothing/a.js"),
-                    code: "var a = (b,) => 1",
+                    code: "var a = { ...obj }",
                     errors: [
                         {
-                            messageId: "no-trailing-function-commas",
-                            data: { supported: "8.0.0", version: ">=6.0.0" },
+                            messageId: "no-rest-spread-properties",
+                            data: { supported: "8.3.0", version: ">=8.0.0" },
                         },
                     ],
                 },
@@ -2478,6 +2615,16 @@ ruleTester.run(
                         },
                     ],
                 },
+                {
+                    filename: fixture("star/a.js"),
+                    code: '"use strict"; let a = 1',
+                    errors: [
+                        {
+                            messageId: "no-block-scoped-variables-strict",
+                            data: { supported: "4.0.0", version: "*" },
+                        },
+                    ],
+                },
                 {
                     code: "var a = async () => 1",
                     options: [{ version: "7.1.0" }],
